import requests
import json
import mutual

default_stream_response = {
                        "error": None,
                        "status": None,
                        "content": None,
                        "bot_id": None,
                        "user_id": None,
                        "new_bot": None,
                        "new_bot_user": None,
                        "prompt_id": None
                        }

def create(content, bot_id=None, username=None, prompt_id=None, error_logs=False, multiplayer_memory = True, context_window = 2):
    if bot_id is None:
        bot_id = mutual.bot_id
    if bot_id is None:
        raise ValueError("bot_id must be provided either as argument or set in config")

    url = "https://api-agent.mutuai.io/api/chat"

    headers = {
        "Content-Type": "application/json",
        "Authorization": f"Bearer {mutual.api_key}"
    }
    data = {
        "content": content,
        "bot_id": str(bot_id),
        "username": username,
        "prompt_id": prompt_id,
        "multiplayer": multiplayer_memory,
        "context_window": context_window
    }

    response = requests.post(url, data=json.dumps(data), headers=headers, stream=True)

    if response.status_code < 300:
        is_new_bot = False
        is_new_user = False
        for line in response.iter_lines():
            if line:  # filter out keep-alive new lines
                json_data = json.loads(line)
                if json_data['error'] is not None and not error_logs:
                    continue
                if json_data['new_bot'] and not is_new_bot:
                    is_new_bot = True
                    print(f"Newly created bot! Here is your id: {json_data['bot_id']}")
                if json_data['new_bot_user'] and not is_new_user:
                    is_new_user = True
                    print(f"New user {json_data['user_id']} created interacting with bot id: {json_data['bot_id']}")
                if json_data['content'] =='[close]':
                    continue
                yield json_data
    else:
        # raise Exception(f"Request failed with status code {response.status_code}, with an Error Message: {json.loads(response.text)['detail']}")
        default_stream_response['content'] = f"Request failed with status code {response.status_code}, with an Error Message: {json.loads(response.text)['detail'] or response.text}"
        return default_stream_response


def create_demo(content, error_logs=False, multiplayer_memory = True, context_window = 2):

    url = "https://api-agent.mutuai.io/api/test_chat"
    headers = {
        "Content-Type": "application/json",
        "Authorization": f"Bearer {mutual.api_key}"
    }
    data = {
        "content": content,
        "multiplayer": multiplayer_memory,
        "context_window": context_window
    }

    response = requests.post(url, data=json.dumps(data), headers=headers, stream=True)

    if response.status_code < 300:
        for line in response.iter_lines():
            if line:  # filter out keep-alive new lines
                json_data = json.loads(line)
                if json_data['error'] is not None and not error_logs:
                    continue
                if json_data['content'] =='[close]':
                    continue
                yield json_data
    else:
        # raise Exception(f"Request failed with status code {response.status_code}, with an Error Message: {json.loads(response.text)['detail']}")
        default_stream_response['content'] = f"Request failed with status code {response.status_code}, with an Error Message: {json.loads(response.text)['detail'] or response.text}"
        return default_stream_response



import requests
import json
import mutual

default_prompt_response = {
    "prompt_id": None,
    "identity_prompt": None,
    "backstory_prompt": None,
    "world_prompt": None,
    "action_prompt": None,
    "internal_thought": None,
    "external_thought": None,
    "internal_thought_memory": None,
    "external_thought_memory": None,
    "summarization_prompt": None,
    "details": "Successful"
}

def get_prompts(limit=20, offset=0):
    url = f"https://api-agent.mutuai.io/api/prompts?limit={limit}&offset={offset}"
    headers = {
        "Authorization": f"Bearer {mutual.api_key}"
    }
    response = requests.get(url, headers=headers)
    if response.status_code < 300:
        return response.json()
    else:
        default_prompt_response["details"] = f"Request failed with status code {response.status_code}, with an Error Message: {json.loads(response.text)['detail'] or response.text}"
        return default_prompt_response

def get_prompt(prompt_id):
    url = f"https://api-agent.mutuai.io/api/prompts/{str(prompt_id)}"
    headers = {
        "Authorization": f"Bearer {mutual.api_key}"
    }
    response = requests.get(url, headers=headers)
    # response.raise_for_status()  # Raise an exception if the response contains an HTTP error status code
    if response.status_code < 300:
        return response.json()
    else:
        default_prompt_response["details"] = f"Request failed with status code {response.status_code}, with an Error Message: {json.loads(response.text)['detail'] or response.text}"
        return default_prompt_response

def create_prompt(
    prompt_id,
    identity_prompt=None,
    backstory_prompt=None,
    world_prompt=None,
    action_prompt=None,
    internal_thought=None,
    external_thought=None,
    internal_thought_memory=None,
    external_thought_memory=None,
    summarization_prompt=None
):
    url = f"https://api-agent.mutuai.io/api/prompts"
    headers = {
        "Content-Type": "application/json",
        "Authorization": f"Bearer {mutual.api_key}"
    }
    data = {
        "prompt_id": prompt_id,
        "identity_prompt": identity_prompt,
        "backstory_prompt": backstory_prompt,
        "world_prompt": world_prompt,
        "action_prompt": action_prompt,
        "internal_thought": internal_thought,
        "external_thought": external_thought,
        "internal_thought_memory": internal_thought_memory,
        "external_thought_memory": external_thought_memory,
        "summarization_prompt": summarization_prompt
    }
    response = requests.post(url, json=data, headers=headers)
    if response.status_code < 300:
        return response.json()
    else:
        default_prompt_response["details"] = f"Request failed with status code {response.status_code}, with an Error Message: {response.text}"
        return default_prompt_response

def update_prompt(prompt_id,
    identity_prompt=None,
    backstory_prompt=None,
    world_prompt=None,
    action_prompt=None,
    internal_thought=None,
    external_thought=None,
    internal_thought_memory=None,
    external_thought_memory=None,
    summarization_prompt=None):

    url = f"https://api-agent.mutuai.io/api/prompts/{str(prompt_id)}"
    headers = {
        "Content-Type": "application/json",
        "Authorization": f"Bearer {mutual.api_key}"
    }
    data = {
        "identity_prompt": identity_prompt,
        "backstory_prompt": backstory_prompt,
        "world_prompt": world_prompt,
        "action_prompt": action_prompt,
        "internal_thought": internal_thought,
        "external_thought": external_thought,
        "internal_thought_memory": internal_thought_memory,
        "external_thought_memory": external_thought_memory,
        "summarization_prompt": summarization_prompt
    }

    # remove keys with None value
    data = {k: v for k, v in data.items() if v is not None}
    response = requests.patch(url, data=json.dumps(data), headers=headers)
    if response.status_code < 300:
        return response.json()
    else:
        default_prompt_response["details"] = f"Request failed with status code {response.status_code}, with an Error Message: {json.loads(response.text)['detail'] or response.text}"
        return default_prompt_response



# __init__.py
from . import Auth, Bot, Chat, Prompt, Dev

api_key = None
bot_id = None


def create_bot(bot_name, bot_org="Mutual", prompt_id="default"):
    # create new bot instance
    global api_key
    global bot_id

    response = Bot.create_bot(bot_name, bot_org, prompt_id)
    if not response['bot_id']:
        print('Failed in creating a bot.')
        raise Exception(f"Something went wrong. Error Message: {response['details']}")
    
    new_bot_instance = Bot.Bot(api_key, response['bot_id'], response['bot_name'], 
                               response['prompt_id'] or "default")
    
    print(f"Successfully Created a new Bot named {bot_name} with an id: {response['bot_id']}")

    bot_id = response['bot_id']
    return new_bot_instance


def generate_bot(bot_id_arg):
    # generate new bot instance
    global api_key
    global bot_id

    response = Bot.get_bot(bot_id_arg)
    if not response['bot_id']:
        print(f'Bot with id: {bot_id_arg} does not exist please create one.')
        raise Exception(f"Something went wrong. Error Message: {response['details']}")
    
    new_bot_instance = Bot.Bot(api_key, response['bot_id'], response['bot_name'], 
                               response['prompt_id'] or "default")
    
    print(f"Successfully Generated Bot named {response['bot_name']} with an id: {bot_id_arg}")

    bot_id = bot_id_arg
    return new_bot_instance


import requests
import json
import mutual

bot_default_response = {
            "bot_id": None,
            "bot_name": None,
            "bot_org": None,
            "bot_chat_index": None,
            "prompt_id": None,
            "details": None
        }

def get_bots(limit=20, offset=0):
    url = f"https://api-agent.mutuai.io/api/bots?limit={limit}&offset={offset}"
    headers = {
        "Authorization": f"Bearer {mutual.api_key}"
    }
    response = requests.get(url, headers=headers)
    if response.status_code < 300:
        return response.json()
    else:
        bot_default_response["details"] = f"Request failed with status code {response.status_code}, with an Error Message: {json.loads(response.text)['detail'] or response.text}"
        return bot_default_response

def get_bot(bot_id):
    url = f"https://api-agent.mutuai.io/api/bots/{str(bot_id)}"
    headers = {
        "Authorization": f"Bearer {mutual.api_key}"
    }
    response = requests.get(url, headers=headers)
    # response.raise_for_status()  # Raise an exception if the response contains an HTTP error status code
    if response.status_code < 300:
        return response.json()
    else:
        bot_default_response["details"] = f"Request failed with status code {response.status_code}, with an Error Message: {json.loads(response.text)['detail'] or response.text}"
        return bot_default_response

def create_bot(bot_name, bot_org = "Mutual", prompt_id = "default"):
    url = "https://api-agent.mutuai.io/api/bots"
    headers = {
        "Content-Type": "application/json",
        "Authorization": f"Bearer {mutual.api_key}"
    }
    data = {
        # "bot_id": str(bot_id),
        "bot_name": bot_name,
        "bot_org": bot_org,
        "prompt_id": prompt_id
    }
    response = requests.post(url, data=json.dumps(data), headers=headers)
    if response.status_code < 300:
        return response.json()
    else:
        bot_default_response["details"] = f"Request failed with status code {response.status_code}, with an Error Message: {json.loads(response.text)['detail'] or response.text}"
        return bot_default_response

def update_bot(bot_id, bot_name=None, bot_org=None, prompt_id=None):
    url = f"https://api-agent.mutuai.io/api/bots/{str(bot_id)}"
    headers = {
        "Content-Type": "application/json",
        "Authorization": f"Bearer {mutual.api_key}"
    }
    data = {
        "bot_name": bot_name,
        "bot_org": bot_org,
        "prompt_id": prompt_id
    }
    # remove keys with None value
    data = {k: v for k, v in data.items() if v is not None}
    response = requests.patch(url, data=json.dumps(data), headers=headers)
    if response.status_code < 300:
        return response.json()
    else:
        bot_default_response["details"] = f"Request failed with status code {response.status_code}, with an Error Message: {json.loads(response.text)['detail'] or response.text}"
        return bot_default_response


class Bot:
    def __init__(self, api_key, bot_id=None, bot_name=None, prompt_id = "default"):
        self.api_key = api_key
        self.bot_id = bot_id
        self.bot_name = bot_name
        self.prompt_id = prompt_id

        self.default_stream_response = {
                        "error": None,
                        "status": None,
                        "content": None,
                        "bot_id": None,
                        "user_id": None,
                        "new_bot": None,
                        "new_bot_user": None,
                        "prompt_id": None
        }

    def updateBot(self, bot_name=None, bot_org=None, prompt_id=None):
        url = f"https://api-agent.mutuai.io/api/bots/{str(self.bot_id)}"
        headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.api_key}"
        }
        data = {
            "bot_name": bot_name,
            "bot_org": bot_org,
            "prompt_id": prompt_id
        }
        # remove keys with None value
        data = {k: v for k, v in data.items() if v is not None}
        response = requests.patch(url, data=json.dumps(data), headers=headers)
        if response.status_code < 300:
            return response.json()
        else:
            self.default_stream_response["content"] = f"Request failed with status code {response.status_code}, with an Error Message: {json.loads(response.text)['detail'] or response.text}"
            return self.default_stream_response

    def chat(self, content, username=None, multiplayer_memory = True, context_window = 2):
        url = "https://api-agent.mutuai.io/api/chat"
        headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.api_key}"
        }
        data = {
            "content": content,
            "bot_id": str(self.bot_id),
            "username": username,
            "prompt_id": self.prompt_id,
            "multiplayer": multiplayer_memory,
            "context_window": context_window
        }

        response = requests.post(url, data=json.dumps(data), headers=headers, stream=True)

        if response.status_code < 300:
            is_new_bot = False
            is_new_user = False
            for line in response.iter_lines():
                if line:  # filter out keep-alive new lines
                    json_data = json.loads(line)
                    if json_data['new_bot'] and not is_new_bot:
                        is_new_bot = True
                        print(f"Newly created bot! Here is your id: {json_data['bot_id']}")
                    if json_data['new_bot_user'] and not is_new_user:
                        is_new_user = True
                        print(f"New user {json_data['user_id']} created interacting with bot id: {json_data['bot_id']}")
                    if json_data['content'] =='[close]':
                        continue
                    yield json_data
        else:
            self.default_stream_response['content'] = f"Request failed with status code {response.status_code}, with an Error Message: {json.loads(response.text)['detail'] or response.text}"
            return self.default_stream_response



import requests
import json
import mutual

def clear(bot_id):
    url = f"https://api-agent.mutuai.io/api/clear/{bot_id}"
    headers = {
        "Authorization": f"Bearer {mutual.api_key}"
    }
    response = requests.get(url, headers=headers)

    if response.status_code < 300:
        response_json = response.json()
        return response_json
    else:
        return {
            "details":f"Request failed with status code {response.status_code}, with an Error Message: {json.loads(response.text)['detail'] or response.text}"
        }


from setuptools import setup, find_packages

setup(
    name='mutual',
    version='0.1.1',  # beta
    description='A Python client for the Mutual API.',
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    author='Alex Betita', # placeholder for now
    author_email='alexbetita25@gmail.com', # placeholder for now
    url='https://github.com/Mutu-AI',  # if you have a github repo for the package
    packages=find_packages(),
    install_requires=[
        'requests'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)



import requests
import json
import mutual

def signup(user_id, email):
    url = "https://api-agent.mutuai.io/api/signup"
    data = {
        "user_id": user_id,
        "email": email
    }

    headers = {
        'Content-Type': 'application/json'
    }

    response = requests.post(url, data=json.dumps(data), headers=headers)

    if response.status_code < 300:
        response_json = response.json()
        mutual.api_key = response_json.get('api_key') # save the api_key to config
        return response_json
    else:
        return {
            "user_id": None,
            "email": None,
            "api_key": None,
            "details":f"Request failed with status code {response.status_code}, with an Error Message: {json.loads(response.text)['detail'] or response.text}"
        }


def login(user_id, email):
    url = "https://api-agent.mutuai.io/api/login"
    data = {
        "user_id": user_id,
        "email": email
    }

    headers = {
        'Content-Type': 'application/json'
    }

    response = requests.post(url, data=json.dumps(data), headers=headers)

    if response.status_code < 300:
        response_json = response.json()
        mutual.api_key = response_json.get('api_key') # save the api_key to config
        return response_json
    else:
        return {
            "user_id": None,
            "email": None,
            "api_key": None,
            "details":f"Request failed with status code {response.status_code}, with an Error Message: {json.loads(response.text)['detail'] or response.text}"
        }